﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Optimization.Operators.LCS {
  [Item("GAssistSpecialStageOperator", "Description missing")]
  [StorableClass]
  public abstract class GAssistSpecialStageOperator : SingleSuccessorOperator, IGAssistSpecialStage {

    #region Parameter Properties
    public ILookupParameter<IGAssistIndividual> IndividualParameter {
      get { return (ILookupParameter<IGAssistIndividual>)Parameters["Individual"]; }
    }
    public ILookupParameter<DoubleValue> ProbabilityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["Probability"]; }
    }
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }
    #endregion

    public IGAssistIndividual Individual {
      get { return IndividualParameter.ActualValue; }
    }
    public double Probability {
      get { return ProbabilityParameter.ActualValue.Value; }
    }
    public IRandom Random {
      get { return RandomParameter.ActualValue; }
    }

    [StorableConstructor]
    protected GAssistSpecialStageOperator(bool deserializing) : base(deserializing) { }
    protected GAssistSpecialStageOperator(GAssistSpecialStageOperator original, Cloner cloner)
      : base(original, cloner) {
    }
    public GAssistSpecialStageOperator()
      : base() {
      Parameters.Add(new LookupParameter<IGAssistIndividual>("Individual"));
      Parameters.Add(new LookupParameter<DoubleValue>("Probability"));
      Parameters.Add(new LookupParameter<IRandom>("Random"));
    }

    public sealed override IOperation Apply() {
      ApplySpecialStage(Individual, Probability, Random);
      return base.Apply();
    }

    protected abstract void ApplySpecialStage(IGAssistIndividual individual, double probability, IRandom random);
  }
}
